#import "FooApplePayPlugin.h"
#import <FooAppleWallet/FOAppleWallet.h>
#import <FooAppleWallet/FOInAppProvisioning.h>

@interface FooApplePayPlugin() <FOInAppProtocol>

@property (nonatomic, strong) FlutterEventSink eventSink;

@end

@implementation FooApplePayPlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {

  //Method Channel
  FlutterMethodChannel* methodChannel = [FlutterMethodChannel
      methodChannelWithName:@"foo_apple_pay"
            binaryMessenger:[registrar messenger]];

  FooApplePayPlugin* instance = [[FooApplePayPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:methodChannel];

    //Event Channel
    FlutterEventChannel* eventChannel = [FlutterEventChannel
       eventChannelWithName:@"foo_apple_pay_events"
           binaryMessenger:[registrar messenger]];
    [eventChannel setStreamHandler:instance];
}

//Method Channel
- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    
  if ([@"setHostNameAndPath" isEqualToString:call.method]) {      
      [self setHostName:call.arguments[0] andPath:call.arguments[1]];      
  } else if ([@"deviceSupportsAppleWallet" isEqualToString:call.method]) {
      result([self deviceSupportsAppleWallet]);
  } else if ([@"getLocalPasses" isEqualToString:call.method]) {
      result([self getLocalPasses]);
  } else if ([@"getRemotePasses" isEqualToString:call.method]) {
      result([self getRemotePasses]);
  } else if ([@"canAddCardWithId" isEqualToString:call.method]) {
      result([self canAddCardWithId:call.arguments[0]]);
  } else if ([@"cardAlreadyAddedToLocalWallet" isEqualToString:call.method]) {
      result([self cardAlreadyAddedToLocalWallet:call.arguments[0]]);
  } else if ([@"cardAlreadyAddedToRemoteWallet" isEqualToString:call.method]) {
      result([self cardAlreadyAddedToRemoteWallet:call.arguments[0]]);
  } else if ([@"addCardForUserIdCardIdCardHolderNameCardPanSuffixLocalizedDescription" isEqualToString:call.method]) {
      [self addCardForUserId:call.arguments[0] cardId:call.arguments[1] cardHolderName:call.arguments[2] cardPanSuffix:call.arguments[3] localizedDescription:call.arguments[4]];
  } else if ([@"addCardForUserIdCardIdCardHolderNameCardPanSuffixLocalizedDescriptionPanExpiryDate" isEqualToString:call.method]) {
      [self addCardForUserId:call.arguments[0] cardId:call.arguments[1] cardHolderName:call.arguments[2] cardPanSuffix:call.arguments[3] localizedDescription:call.arguments[4] pan:call.arguments[5] expiryDate:call.arguments[6]];
  } else {
    result(FlutterMethodNotImplemented);
  }
}

- (void)setHostName:(NSString *)hostName andPath:(NSString *)path
{
    [FOAppleWallet setHostName:hostName andPath:path];
}

- (NSNumber *)deviceSupportsAppleWallet
{
    BOOL result = [FOInAppProvisioning deviceSupportsAppleWallet];
    return [NSNumber numberWithBool:result];
}

- (nullable NSArray <PKPass *>*)getLocalPasses
{
    return [FOInAppProvisioning getLocalPasses];
}

- (nullable NSArray <PKPaymentPass *>*)getRemotePasses
{
    return [FOInAppProvisioning getRemotePasses];
}

- (NSNumber *)canAddCardWithId:(NSString *)cardId
{
    BOOL result = [FOInAppProvisioning canAddCardWithId:cardId];
    return [NSNumber numberWithBool:result];
}

- (NSNumber *)cardAlreadyAddedToLocalWallet:(NSString *)cardId
{
    BOOL result = [FOInAppProvisioning cardAlreadyAddedToLocalWallet:cardId];
    return [NSNumber numberWithBool:result];
}

- (NSNumber *)cardAlreadyAddedToRemoteWallet:(NSString *)cardId
{
    BOOL result = [FOInAppProvisioning cardAlreadyAddedToLocalWallet:cardId];
    return [NSNumber numberWithBool:result];
}

- (void)addCardForUserId:(nullable NSString *)userId cardId:(NSString *)cardId cardHolderName:(NSString *)cardHolderName cardPanSuffix:(NSString *)cardPanSuffix localizedDescription:(nullable NSString *)localizedDescription
{
    return [FOInAppProvisioning addCardForUserId:userId cardId:cardId cardHolderName:cardHolderName cardPanSuffix:cardPanSuffix localizedDescription:localizedDescription inViewController:[UIApplication sharedApplication].keyWindow.rootViewController delegate:self];
}

- (void)addCardForUserId:(nullable NSString *)userId cardId:(NSString *)cardId cardHolderName:(NSString *)cardHolderName cardPanSuffix:(NSString *)cardPanSuffix localizedDescription:(nullable NSString *)localizedDescription pan:(NSString *)pan expiryDate:(NSString *)expiryDate
{
    return [FOInAppProvisioning addCardForUserId:userId cardId:cardId cardHolderName:cardHolderName cardPanSuffix:cardPanSuffix localizedDescription:localizedDescription pan:pan expiryDate:expiryDate inViewController:[UIApplication sharedApplication].keyWindow.rootViewController delegate:self];
}

//Event Channel
- (FlutterError *)onListenWithArguments:(id)arguments eventSink:(FlutterEventSink)eventSink {
    self.eventSink = eventSink;
    return nil;
}

- (FlutterError *)onCancelWithArguments:(id)arguments {
    self.eventSink = nil;
    return nil;
}

- (void)didFinishAddingCard:(nullable PKPaymentPass *)pass error:(FOInAppAddCardError)error errorMessage:(nullable NSString *)errorMessage {    
    [[[NSOperationQueue alloc] init] addOperationWithBlock:^{
      if(self.eventSink != nil) {
         if (error == FOInAppAddCardErrorNone) {
             //Convert Pass to Json Object
             self.eventSink(pass);
         } else {
           self.eventSink([FlutterError errorWithCode:[NSString stringWithFormat:@"%d", error]
                                          message:errorMessage
                                          details:nil]);
         }
      }
    }];
}

@end
